/*
 * plpcfgbt.c - configure the binary Plop Boot Manager version 
 *
 * Released under GPL v2
 * by Elmar Hanlhofer  http://www.plop.at
 *
 */


#define VERSION "0.11 20120203"

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef WIN32
#include "windows.h"
#include "winioctl.h"
#endif

#include <stdlib.h>
#include <string.h>


#define BT_MAJOR 	0x284
#define BT_MINOR 	0x285
#define BT_EXTRA 	0x286
#define CONFIG_VERSION 	0x287
#define CONFIG_OFFSET 	0x288

#define CONFIG_VERSION_NUMBER 9

// this is not the complete configuration structure, only the required 
// for this program
typedef struct _config {

    char videoMode;
    char lastProfile;
    char useDefaultProfile;
    char defaultProfile;
    char bootCounter;
    char bootCounterValue;
    char floppyBoot;
    char cdromBoot;
    char usbBoot;
    char hideSetup;
    char numlock;
    char startModeHidden;
    char textmodeAfterBoot;
    char stars;
    char dateformat;
    char masterpassword;
    char setuppassword;
    char font;
    char zoomanimation;
    char dummy1;
    char dummy2;
    char dummy3;
    char dummy4;
    char forceUSB1;
    char useint19h;
    char usbmdev;
    char hotkey;
    char netBoot;
    char skippcmcia;
    char fastpcmcia;
    char ignoreusbdevs;
    char showExtended;
    char dummy5[432];
    char bootPartition[16];
    char bootDevice[16];
    char dummy6[16 +16 * 16 + 16 * 16];
    char profileLabel[16][16];
} _config;


char VIDEOMODE[][10] = { "text mode",
		    "640x480",
		    "800x600",
		    "1024x768",
		    "1280x1024"
		 };
		 
char STARTMODE[][10] = { "menu",
		    "hidden"
		  };

char ONOFF[][10] = { "off",
		"on"
	      };

char MODE[][10] = { "off",
		"1",
		"2"
	      };

char DEFAULTBOOT[][10] = { "off",
		      "profile",
		      "floppy",
		      "cdrom",
		      "usb"
	    	    };

char FONT[][20] = { "bootmanager",
	       "bios"
	     };

char YESNO[][10] = { "no",
		"yes"
	      };


void printHelp()
{

    printf ("plpcfgbt " VERSION " by Elmar Hanlhofer http://www.plop.at\n\n");
    printf ("Configure the plop boot manager binary.\n\n");
    printf ("Usage: plpcfgbt [OPTIONS] file\n\n");
    printf ("Options:\n");
    printf ("========\n\n");

    printf (" parameter    meaning\n");
    printf (" -----------------------------------------------------------------------------\n");
    printf (" vm           setup video mode\n");
    printf ("              values: text, 640x480, 800x600, 1024x768, 1280x1024\n");
    printf ("\n");
    printf (" stm          start mode of the boot manager\n");
    printf ("              values: menu or hidden, hidden requires cnt=on and dbt not off\n");
    printf ("\n");
    printf (" cnt          activate countdown, values: on or off\n");
    printf ("\n");
    printf (" cntval       countdown in seconds between 1 and 99\n");
    printf ("\n");
    printf (" dbt          default boot, values: off, profile, floppy, cdrom, usb\n");
    printf ("\n");
    printf (" dprf         use default profile value between 1 and 16\n");
    printf ("\n");
    printf (" stf          starfield, values: on or off\n");
    printf ("\n");
    printf (" zan          windows zoom animation, values: on or off\n");
    printf ("\n");
    printf (" fnt          set font, values: bt or bios\n");
    printf ("\n");
    printf (" usb1         force usb 1.1\n");
    printf ("              values:   off\n");
    printf ("                        1 = Mode 1 = skip ehci\n");
    printf ("                        2 = Mode 2 = force ehci to companion\n");
    printf ("\n");
    printf (" usbmdev      use usb massdev 1-99 (not available for 5.0.12)\n");
    printf ("\n");
    printf (" int19h       do not boot the os and use int19h, values: on or off\n");
    printf ("\n");
    printf (" hotkey       startup hotkey, values: yes or no\n");
    printf ("\n");
    printf (" prf          set profile, value: 1-16\n");
    printf ("\n");
    printf (" prfname      set profilename 16 chars, 'prf' must be set\n");
    printf ("\n");
    printf (" btdrv        boot drive, 80h-89h, 'prf' must be set\n");
    printf ("\n");
    printf (" btprt        boot partition 1-4, 'prf' must be set\n");
    printf ("\n");
    printf (" btlog        boot log partition, 1-4, 'prf' must be set\n");
    printf ("\n");
    printf (" skippcmcia   skip pcmcia check, values: yes or no\n");
    printf ("\n");
    printf (" fastpcmcia   skip pcmcia check after usb card was found, values: yes or no\n");
    printf ("\n");
    printf (" ignoreusbdev ignore number of usb devices 0-99 \n");
    printf ("\n");
    printf (" showextended show extended partitions in the main menu, values: yes or no\n");
    printf ("\n");
    printf ("\n");
    printf (" hiddenusb    automatic usb boot without showing the boot manager\n");
    printf (" hiddencdrom  automatic cdrom boot without showing the boot manager\n");
    printf ("\n");
    printf ("\n");
    printf ("Examples:\n");
    printf ("=========\n\n");
    printf (" Print current values: plpcfgbt plpbt.bin\n");
    printf (" Hidden boot with usb: plpcfgbt stm=hidden cnt=on cntval=1 dbt=usb plpbt.bin\n");
    printf ("                   or  plpcfgbt hiddenusb plpbt.bin\n");
    printf ("\n");
#ifdef WIN32
    printf (" **** Use 'plpcfgbt | more' to see the whole screen\n\n");
#else
    printf (" **** Use 'plpcfgbt | less' to see the whole screen\n\n");
#endif

}


void printInfos(_config *conf)
{
    
    printf ("Video mode     : %s\n",VIDEOMODE[conf->videoMode]);
    printf ("Start mode     : %s\n",STARTMODE[conf->startModeHidden]);
    printf ("Countdown      : %s\n",ONOFF[conf->bootCounter]);
    printf ("Countdown value: %d seconds\n",conf->bootCounterValue);
    printf ("Default boot   : %s\n",DEFAULTBOOT[conf->useDefaultProfile]);
    printf ("Profile        : %d\n",conf->defaultProfile+1);
    printf ("Starfield      : %s\n",ONOFF[conf->stars]);
    printf ("Zoom animation : %s\n",ONOFF[conf->zoomanimation]);
    printf ("Font           : %s\n",FONT[conf->font]);
    printf ("Force USB 1.1  : %s\n",MODE[conf->forceUSB1]);
    printf ("Use USB MassDev: %d\n",conf->usbmdev+1);
    printf ("Use INT19h     : %s\n",ONOFF[conf->useint19h]);
    printf ("Startup hotkey : %s\n",YESNO[conf->hotkey]);
    printf ("Skip PCMCIA    : %s\n",YESNO[conf->skippcmcia]);
    printf ("Fast PCMCIA    : %s\n",YESNO[conf->fastpcmcia]);
    printf ("Ignore USB devs: %d\n",conf->ignoreusbdevs);
    printf ("Show extended p: %s\n",YESNO[conf->showExtended]);
}

int param (__const char *t, char *val, char *arg)
{
    if (strncmp (arg,t,strlen (t)) == 0) {
	strcpy (val,arg+strlen(t));
	return 1;
    }

    return 0;
}

//=================================================================

int main (int argc, char *argv[])
{

    char filebuffer[64*1024];
    char filename[1024];
    int filesize;
    FILE *pf;
    char val[255];
    int ival;
    
    int profile = -1;
    int bootdrv = -1;
    int bootprt = -1;
    int bootlog = -1;
    char prflabel[16];
    int setlabel=0;

    _config *conf;

    int i;

    if (argc == 1) {
	printHelp();
	return 0;    
    }
    
    strcpy (filename,argv[argc-1]);
    

    //--------------------
    // read file
    //--------------------
    if (!(pf = fopen (filename,"rb"))) 
    {
	printf ("Error: unable to open %s\n",filename);
	return 1;    
    }
    
    fseek (pf,0,SEEK_END);
    filesize = ftell (pf);
    fseek (pf,0,SEEK_SET);
    
    if (!fread (filebuffer,filesize,1,pf)) 
    {
	printf ("Error: unable to read file\n");
	return 1;        
    }
    
    fclose (pf);
    

    
    conf = (struct _config *)(filebuffer+CONFIG_OFFSET);
    
    //--------------------
    // printf currrent configuration
    //--------------------
    if (argc == 2) 
    {
	printInfos (conf);
	return 0;
    }

    //--------------------
    // check config version
    //--------------------
    if (filebuffer[CONFIG_VERSION] < CONFIG_VERSION_NUMBER) 
    {
	printf ("Warning: %s has old config stucture, maybe some configurations\n",filename);
	printf ("         have no effect.\n\n");
    }

    //--------------------
    // parameter stuff
    //--------------------
    for (i = 1; i < argc-1; i++)
	if (param("vm=",val,argv[i])) 
	{
	    if (strcmp (val,"text") == 0) conf->videoMode = 0;
	    else if (strcmp (val,"640x480") == 0) conf->videoMode = 1;
	    else if (strcmp (val,"800x600") == 0) conf->videoMode = 2;
	    else if (strcmp (val,"1024x768") == 0) conf->videoMode = 3;
	    else if (strcmp (val,"1280x1024") == 0) conf->videoMode = 4;
	    else {
		printf ("Parameter error: vm, run plpcfgbt without parameter to see help\n");
		return 1;
	    }
	}

    for (i = 1; i < argc-1; i++)
	if (param("cnt=",val,argv[i]))
	    if (strcmp (val,"on") == 0)
		conf->bootCounter = 1;
	    else if (strcmp (val,"off") == 0)
		conf->bootCounter = 0;
	    else {
		printf ("Parameter error: cnt, only on and off is valid\n");
		return 1;
	    }


    for (i = 1; i < argc-1; i++)
	if (param("cntval=",val,argv[i])) 
	{
	    ival = atoi (val);
	    if ((ival < 1) || (ival > 99)) 
	    {
		printf ("Parameter error: cntval, only values 1-99 are valid\n");
		return 1;
	    } else conf->bootCounterValue = ival;
	}

    for (i = 1; i < argc-1; i++)
	if (param("usbmdev=",val,argv[i])) 
	{
	    ival = atoi (val);
	    if ((ival < 1) || (ival > 99)) 
	    {
		printf ("Parameter error: usbmdev, only values 1-99 are valid\n");
		return 1;
	    } else conf->usbmdev = ival-1;
	}

    for (i = 1; i < argc-1; i++)
	if (param("ignoreusbdev=",val,argv[i])) 
	{
	    ival = atoi (val);
	    if ((ival < 0) || (ival > 99)) 
	    {
		printf ("Parameter error: ignoreusbdev, only values 0-99 are valid\n");
		return 1;
	    } else conf->ignoreusbdevs = ival;
	}

    for (i = 1; i < argc-1; i++)
	if (param("dbt=",val,argv[i])) 
	{
	    if (strcmp (val,"off") == 0) conf->useDefaultProfile = 0;
	    else if (strcmp (val,"profile") == 0) conf->useDefaultProfile = 1;
	    else if (strcmp (val,"floppy") == 0) conf->useDefaultProfile = 2;
	    else if (strcmp (val,"cdrom") == 0) conf->useDefaultProfile = 3;
	    else if (strcmp (val,"usb") == 0) conf->useDefaultProfile = 4;
	    else {
		printf ("Parameter error: dbt, run plpcfgbt without parameter to see help\n");
		return 1;
	    }
	}


    for (i = 1; i < argc-1; i++)
	if (param("stm=",val,argv[i]))
	    if ((strcmp (val,"hidden") != 0) &&
		(strcmp (val,"menu") != 0)) 
	    {
		printf ("Parameter error: stm, menu or hidden alowed\n");
		return 1;
	    } else if ((!conf->bootCounter || !conf->useDefaultProfile) && 
	        (strcmp (val,"hidden") == 0)) {
		printf ("Parameter error: stm, hidden mode requires countdown and default boot\n");
		return 1;
	    } else if (strcmp (val,"menu") == 0) conf->startModeHidden = 0;
	    else if (strcmp (val,"hidden") == 0)  conf->startModeHidden = 1;

    for (i = 1; i < argc-1; i++)
	if (param("dprf=",val,argv[i])) 
	{
	    ival = atoi (val);
	    if ((ival < 1) || (ival > 16)) 
	    {
		printf ("Parameter error: dprf, only values 1-16 are valid\n");
		return 1;
	    } else conf->defaultProfile = ival - 1;
	}


    for (i = 1; i < argc-1; i++)
	if (param("stf=",val,argv[i]))
	    if (strcmp (val,"on") == 0)
		conf->stars = 1;
	    else if (strcmp (val,"off") == 0)
		conf->stars = 0;
	    else {
		printf ("Parameter error: stf, only on and off is valid\n");
		return 1;
	    }

    for (i = 1; i < argc-1; i++)
	if (param("zan=",val,argv[i]))
	    if (strcmp (val,"on") == 0)
		conf->zoomanimation = 1;
	    else if (strcmp (val,"off") == 0)
		conf->zoomanimation = 0;
	    else {
		printf ("Parameter error: zan, only on and off is valid\n");
		return 1;
	    }

    for (i = 1; i < argc-1; i++)
	if (param("fnt=",val,argv[i]))
	    if (strcmp (val,"bt") == 0)
		conf->font = 0;
	    else if (strcmp (val,"bios") == 0)
		conf->font = 1;
	    else {
		printf ("Parameter error: fnt, only bt and bios is valid\n");
		return 1;
	    }

    for (i = 1; i < argc-1; i++)
	if (param("usb1=",val,argv[i]))
	    if (strcmp (val,"off") == 0)
		conf->forceUSB1 = 0;
	    else if (strcmp (val,"1") == 0)
		conf->forceUSB1 = 1;
	    else if (strcmp (val,"2") == 0)
		conf->forceUSB1 = 2;
	    else {
		printf ("Parameter error: usb1, only off, 1, 2 is valid\n");
		return 1;
	    }

    for (i = 1; i < argc-1; i++)
	if (param("int19h=",val,argv[i]))
	    if (strcmp (val,"on") == 0)
		conf->useint19h = 1;
	    else if (strcmp (val,"off") == 0)
		conf->useint19h = 0;
	    else {
		printf ("Parameter error: int19h, only on and off is valid\n");
		return 1;
	    }

    for (i = 1; i < argc-1; i++)
	if (param("hotkey=",val,argv[i]))
	    if (strcmp (val,"yes") == 0)
		conf->hotkey = 1;
	    else if (strcmp (val,"no") == 0)
		conf->hotkey = 0;
	    else {
		printf ("Parameter error: hk, only yes and no is valid\n");
		return 1;
	    }
	    

    for (i = 1; i < argc-1; i++)
	if (param("prf=",val,argv[i])) 
	{
	    ival = atoi (val);
	    if ((ival < 1) || (ival > 16)) 
	    {
		printf ("Parameter error: prf, only values 1-16 are valid\n");
		return 1;
	    } else profile = ival;
	}

    for (i = 1; i < argc-1; i++)
	if (param("prfname=",val,argv[i])) 
	{
	    int c;
	    int l = strlen (val);
	    if (l > 16) l = 16;
	    
	    for (c = 0; c < 16; c++)
		prflabel[c]=0;
		
	    for (c = 0; c < l; c++)
		prflabel[c]=val[c];
	
	    setlabel = 1;
		
	}

    for (i = 1; i < argc-1; i++)
	if (param("btdrv=",val,argv[i])) 
	{

	    if ((strlen (val) != 3) || (val[0] != '8') || (val[2] != 'h') ||
	        (val[1] < '0') || (val[1] > '9'))
	    {
		printf ("Parameter error: btdrv, only values 80h-89h are valid\n");
		return 1;
	    } else bootdrv = val[1];
		
	}

    for (i = 1; i < argc-1; i++)
	if (param("btprt=",val,argv[i])) 
	{
	    ival = atoi (val);
	    
	    if ((ival < 1) || (ival > 4))
	    {
		printf ("Parameter error: btprt, only values 1-4 are valid\n");
		return 1;
	    }
	    else bootprt = ival;
		
	}

    for (i = 1; i < argc-1; i++)
	if (param("btlog=",val,argv[i])) 
	{
	    ival = atoi (val);
	    
	    if ((ival < 1) || (ival > 4))
	    {
		printf ("Parameter error: btlog, only values 1-4 are valid\n");
		return 1;
	    }
	    else bootlog = ival;
		
	}

    for (i = 1; i < argc-1; i++)
	if (param("skippcmcia=",val,argv[i]))
	    if (strcmp (val,"yes") == 0)
		conf->skippcmcia = 1;
	    else if (strcmp (val,"no") == 0)
		conf->skippcmcia = 0;
	    else {
		printf ("Parameter error: skippcmcia, only yes and no is valid\n");
		return 1;
	    }
	    

    for (i = 1; i < argc-1; i++)
	if (param("fastpcmcia=",val,argv[i]))
	    if (strcmp (val,"yes") == 0)
		conf->fastpcmcia = 1;
	    else if (strcmp (val,"no") == 0)
		conf->fastpcmcia = 0;
	    else {
		printf ("Parameter error: fastpcmcia, only yes and no is valid\n");
		return 1;
	    }
	    
    for (i = 1; i < argc-1; i++)
	if (param("showextended=",val,argv[i]))
	    if (strcmp (val,"yes") == 0)
		conf->showExtended = 1;
	    else if (strcmp (val,"no") == 0)
		conf->showExtended = 0;
	    else {
		printf ("Parameter error: showextended, only yes and no is valid\n");
		return 1;
	    }
	    

    for (i = 1; i < argc-1; i++)
	if (param("hiddencdrom",val,argv[i]))
	{
	    conf->startModeHidden = 1;
	    conf->bootCounter = 1;
	    conf->bootCounterValue = 1;
	    conf->useDefaultProfile = 3;
	}

    for (i = 1; i < argc-1; i++)
	if (param("hiddenusb",val,argv[i]))
	{
	    conf->startModeHidden = 1;
	    conf->bootCounter = 1;
	    conf->bootCounterValue = 1;
	    conf->useDefaultProfile = 4;
	}



    //--------------------
    // write back to file
    //--------------------
    
    if (profile != -1) 
    {
	profile--;
	if (setlabel) 
	{
//printf ("ffff");
	    for (i = 0; i < 16 ; i++)
		conf->profileLabel[profile][i] = prflabel[i];
	}
	
	if (bootdrv != -1)
	    conf->bootDevice[profile] = bootdrv + 0x80 - '0';

	if (bootprt != -1)
	    conf->bootPartition[profile] = (conf->bootPartition[profile] & 0xE0) 
					    + bootprt;

	if (bootlog != -1)
	    conf->bootPartition[profile] = (conf->bootPartition[profile] & 0x1F) 
					    + (bootlog << 5);

    }
    
    

  
  
    if (!(pf = fopen (filename,"wb"))) 
    {
	printf ("Error: unable to open %s for writing\n",filename);
	return 1;    
    }
    
    if (!fwrite (filebuffer,filesize,1,pf)) 
    {
	printf ("Error: unable to write to file\n");
	return 1;        
    }
    
    fclose (pf);

    printf ("data changed\n\n");    

#ifdef WIN32
    printf ("hint: if you use the windows boot menu to start the boot manager, then its\n");
    printf ("a good idea to run again plpbt4win or the batch file!\n");

#endif

    return 0;
}
